﻿using System;
using System.Collections;
using System.Collections.Generic;

namespace IEnumerableImplementation {

	class A : IEnumerable<int> {
		public IEnumerator<int> GetEnumerator() {
			Console.WriteLine("A.GetEnumerator()");
			return new Enumerator();
		}

		IEnumerator IEnumerable.GetEnumerator() {
			return this.GetEnumerator();
		}

		private class Enumerator : IEnumerator<int> {
			public int Current { get; private set; }

			object IEnumerator.Current {
				get {
					return this.Current;
				}
			}

			public bool MoveNext() {
				if (Current <= 5) {
					Current++;
					Console.WriteLine("AEnumerator.MoveNext() -> true");
					return true;
				} else {
					Console.WriteLine("AEnumerator.MoveNext() -> false");
					return false;
				}
			}

			public void Reset() {
				Current = 0;
			}

			public void Dispose() {
			}
		}
	}

	class Program {
		static void Main(string[] args) {
			var a = new A();

			Console.WriteLine("before GetEnumerator()");
			var e = a.GetEnumerator();
			while (e.MoveNext()) {
				Console.WriteLine(e.Current);
			}
			Console.WriteLine("after while");

			Console.WriteLine();

			Console.WriteLine("before foreach");
			foreach (var i in a) {
				Console.WriteLine(i);
			}
			Console.WriteLine("after foreach");
		}
	}
}
