﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace EnumerableRandom {

	class RandomEnumerable : IEnumerable<int> {
		private Random _random;
		private int _from, _to;

		public RandomEnumerable(int from, int to) {
			_random = new Random(123);
			_from = from;
			_to = to;
		}

		public IEnumerator<int> GetEnumerator() => new Enumerator(this);

		IEnumerator IEnumerable.GetEnumerator() => GetEnumerator();

		private class Enumerator : IEnumerator<int> {
			private RandomEnumerable _targetEnumerable;

			public Enumerator(RandomEnumerable target) {
				_targetEnumerable = target;
			}

			public bool MoveNext() {
				Current = _targetEnumerable._random.Next(_targetEnumerable._from, _targetEnumerable._to);
				return true;
			}

			public int Current { get; private set; }
			object IEnumerator.Current => Current;

			public void Reset() { }

			public void Dispose() { }
		}
	}

	class Program {
		static void PrintFirst5(IEnumerable<int> list) {
			int i = 1;
			var enumerator = list.GetEnumerator();
			while (enumerator.MoveNext()) {
				Console.WriteLine(enumerator.Current);
				if (i >= 5) {
					break;
				}
				i++;
			}
			Console.WriteLine("-----");
		}

		static void Main(string[] args) {
			var list = new List<int> { 1, 2, 3, 4, 5, 6, 7, 8, 9, 10 };
			var random = new RandomEnumerable(10, 100);

			PrintFirst5(list);
			PrintFirst5(random);

			Console.WriteLine("###########");

			PrintUntilDivisibleBy8(list);
			PrintUntilDivisibleBy8(random);
		}

		static void PrintUntilDivisibleBy8(IEnumerable<int> list) {
			foreach (var x in list) {
				Console.WriteLine(x);
				if (x % 8 == 0) break;
			}
			Console.WriteLine("-----");
		}
	}
}
